/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <cstdio>
#include <cstdlib>
#include "conf.h"
#include "esdout.h"
#include "oss.h"
#include "fftcalc.h"

namespace Tuner {
	const CalcInfo DefCalcInfo = { 32768, Window::NONE,
		PEAK_SEARCH, false, 0, 0, 0, 0 };

	const NoteInfo DefNoteInfo = { ENGLISH, 0, 440.0 };

	const char *CalcKeyVal[8] = { "CalcPoints", "WindowType", "CalcMode", "UseGate",
		"GatePercent", "GateHold", "GateRelease", "GateAttack" };

	const char *NoteKeyVal[2] = { "NamingConvention", "RefFreq" };
}

void
Tuner::char2int(const unsigned char *ibuf, int *obuf, int len, ByteOrder order) throw()
{
	int i;
	len /= 2;
	switch(order) {
		case LITTLEENDIAN:
			for(i = 0; i < len; ++i)
				obuf[i] = (ibuf[i * 2] |
				(static_cast<int>(ibuf[i * 2 + 1]) << 8)) - 32768;
			break;
		case BIGENDIAN:
			for(i = 0; i < len; ++i)
				obuf[i] = (ibuf[i * 2 + 1] |
				(static_cast<int>(ibuf[i * 2]) << 8)) - 32768;
	}
}

bool
Tuner::CalcInfo::load(Config::ConfFile &cf) throw(Error::ConfError)
{
	std::string val[8];
	int iVal[8];

	for(int i = 0; i < 8; ++i)
		cf.getValue(CalcKeyVal[i], val[i]),
			iVal[i] = std::atoi(val[i].c_str());

	if(val[0] != "" && iVal[0] > 0)
		points = iVal[0];

	if(val[1] != "" && iVal[1] >= Window::NONE && iVal[1] <= Window::BLACKMAN)
		wtype = Window::WindowType(iVal[1]);

	if(val[2] != "" && iVal[2] >= MAX_SEARCH && iVal[2] <= PEAK_SEARCH)
		mode = FftCalcMode(iVal[2]);

	if(val[3] != "" && iVal[3] >= 0 && iVal[3] <= 1)
		useGate = bool(iVal[3]);

	if(val[4] != "" && iVal[4] >= 0)
		percent = iVal[4];
	if(val[5] != "" && iVal[5] >= 0)
		hold = iVal[5];
	if(val[6] != "" && iVal[6] >= 0)
		rel = iVal[6];
	if(val[7] != "" && iVal[7] >= 0)
		att = iVal[7];

	return true;
}

bool
Tuner::CalcInfo::loadDefaults() throw()
{
	*this = DefCalcInfo;
	return true;
}

bool
Tuner::CalcInfo::save(Config::ConfFile &cf) throw(Error::ConfError)
{
	char buf[8][25];
	std::sprintf(buf[0], "%d", points); 
	std::sprintf(buf[1], "%d", wtype);
	std::sprintf(buf[2], "%d", mode);
	std::sprintf(buf[3], "%d", useGate);
	std::sprintf(buf[4], "%d", percent);
	std::sprintf(buf[5], "%d", hold);
	std::sprintf(buf[6], "%d", rel);
	std::sprintf(buf[7], "%d", att);

	for(int i = 0; i < 8; ++i)
		cf.setValue(CalcKeyVal[i], buf[i]);

	return true;
}

bool
Tuner::NoteInfo::load(Config::ConfFile &cf) throw(Error::ConfError)
{
	std::string val[2];
	int nc;
	double r;

	for(int i = 0; i < 2; ++i)
		cf.getValue(NoteKeyVal[i], val[i]);

	nc = std::atoi(val[0].c_str());
	r = std::atof(val[1].c_str());

	if(val[0] != "" && nc >= ENGLISH && nc <= ITALIAN)
		conv = NamingConv(nc);

	if(val[1] != "" && r > 0)
		ref = r;

	return true;
}

bool
Tuner::NoteInfo::loadDefaults() throw()
{
	*this = DefNoteInfo;
	return true;
}

bool
Tuner::NoteInfo::save(Config::ConfFile &cf) throw(Error::ConfError)
{
	char buf[2][50];

	std::sprintf(buf[0], "%d", conv);
	std::sprintf(buf[1], "%f", ref);

	for(int i = 0; i < 2; ++i)
		cf.setValue(NoteKeyVal[i], buf[i]);

	return true;
}

bool
Tuner::Conf::setInfo(AudioDriver ad, DetectionMode d,
	const AudioInfo *ai, const CalcInfo *fi, const NoteInfo *ni) throw()
{
	audr = ad, dm = d;
	if(ai)
		audio = *ai;
	else
		audio.loadDefaults();

	if(fi)
		fft = *fi;
	else
		fft.loadDefaults();

	if(ni)
		note = *ni;
	else
		note.loadDefaults();

	update();

	return true;
}

void
Tuner::Conf::update() throw()
{
	note.delta = static_cast<double>(audio.rate) /
		static_cast<double>(fft.points);
}

bool
Tuner::Conf::sync() throw()
{
	Note::setNamingConv(note.conv);
	Note::setRef(note.ref);
	Note::setDelta(note.delta);
}

Tuner::Audio *
Tuner::Conf::createAudio() throw(Error::AudioError)
{
	if(audioPtr)
		delete audioPtr;

	audio.mode = RECORD;
	audio.fragNumber = 512;
	audio.fragSize = 256;
	audio.chn = MONO;

	switch(audr) {
		case OSS_DRIVER:
			return audioPtr = new Oss();
		case ESD_DRIVER:
			return audioPtr = new Esd();
	}
}

Tuner::Calc *
Tuner::Conf::createCalc() throw(Error::CalcError)
{
	if(calcPtr)
		delete calcPtr;

	switch(dm) {
		case FFTCALC:
			return calcPtr = new FftCalc(fft.points, audio.rate, audio.fmt,
				fft.wtype,	fft.mode, fft.useGate, fft.percent, fft.hold,
				fft.rel, fft.att);
	}
}

unsigned char *
Tuner::Conf::createBuf() throw(std::bad_alloc)
{
	if(bufPtr)
		delete []bufPtr;

	bufLen = fft.points * (audio.fmt == U8 ? 1: 2);
	return bufPtr = new unsigned char[bufLen];
}

int
Tuner::Conf::getBufSize() throw()
{
	return bufLen = fft.points * (audio.fmt == U8 ? 1: 2);
}

void
Tuner::Conf::destroy() throw()
{
	if(audioPtr)
		delete audioPtr;
	if(calcPtr)
		delete calcPtr;
	if(bufPtr)
		delete []bufPtr;
}

