/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 * rewrite handling of audr e dm
 */

#ifndef _CONF_H_
#define _CONF_H_

#include <new>
#include "audio.h"
#include "calc.h"
#include "note.h"
#include "window.h"
#include "calcerror.h"
#include "conferror.h"
#include "conffile.h"

namespace Tuner {
	enum ByteOrder {
		LITTLEENDIAN,
		BIGENDIAN
	};

	enum FftCalcMode {
		MAX_SEARCH,
		PEAK_SEARCH
	};

	void char2int(const unsigned char *, int *, int, ByteOrder) throw();

	enum AudioDriver {
		OSS_DRIVER,
		ESD_DRIVER
	};

	enum DetectionMode {
		FFTCALC
	};

	struct CalcInfo {
		int points;
		Window::WindowType wtype;
		FftCalcMode mode;
		bool useGate;
		int percent;
		int hold;
		int rel;
		int att;

		bool load(Config::ConfFile &) throw(Error::ConfError);
		bool loadDefaults() throw();
		bool save(Config::ConfFile &) throw(Error::ConfError);
	};

	struct NoteInfo {
		NamingConv conv;
		double delta;
		double ref;

		bool load(Config::ConfFile &) throw(Error::ConfError);
		bool loadDefaults() throw();
		bool save(Config::ConfFile &) throw(Error::ConfError);
	};

	class Conf {
		public:
			Conf(AudioDriver, DetectionMode, AudioInfo * = 0,
				CalcInfo * = 0, NoteInfo * = 0) throw();
		   ~Conf();

		bool load(Config::ConfFile &) throw(Error::ConfError, Error::AudioError);
		bool loadDefaults() throw();
		bool save(Config::ConfFile &) throw(Error::ConfError, Error::AudioError);

		bool setInfo(AudioDriver, DetectionMode, const AudioInfo *,
			const CalcInfo *, const NoteInfo *) throw();

		void update() throw();

		bool sync() throw();

		AudioInfo &getAudioInfo() throw();
		CalcInfo &getCalcInfo() throw();
		NoteInfo &getNoteInfo() throw();

		AudioDriver &getAudioDriver() throw();
		DetectionMode &getDetectionMode() throw();

		Audio *createAudio() throw(Error::AudioError);
		Calc *createCalc() throw(Error::CalcError);
		unsigned char *createBuf() throw(std::bad_alloc);
		int getBufSize() throw();

		void destroy() throw();

		protected:
		AudioInfo audio;
		CalcInfo fft;
		NoteInfo note;

		AudioDriver audr;
		DetectionMode dm;

		private:
		Audio *audioPtr;
		Calc *calcPtr;
		unsigned char *bufPtr;
		int bufLen;
	};

	inline Conf::Conf(AudioDriver ad, DetectionMode d,
		AudioInfo *ai, CalcInfo *fi, NoteInfo *ni) throw():
		audioPtr(0), calcPtr(0), bufPtr(0), bufLen(0)
	{ setInfo(ad, d, ai, fi, ni); }
	inline Conf::~Conf() { destroy(); }
	inline bool Conf::load(Config::ConfFile &f) throw(Error::ConfError, Error::AudioError)
	{ audio.load(f), fft.load(f), note.load(f), update(); return true; }
	inline bool Conf::loadDefaults() throw()
	{ audio.loadDefaults(), fft.loadDefaults(), note.loadDefaults(), update();
		return true; }
	inline bool Conf::save(Config::ConfFile &f) throw(Error::ConfError, Error::AudioError)
	{ audio.save(f), fft.save(f), note.save(f); return true; }

	inline AudioInfo &
	Conf::getAudioInfo() throw() { return audio; }
	inline CalcInfo &
	Conf::getCalcInfo() throw() { return fft; }
	inline NoteInfo &
	Conf::getNoteInfo() throw() { return note; }
	inline AudioDriver &
	Conf::getAudioDriver() throw() { return audr; }
	inline DetectionMode &
	Conf::getDetectionMode() throw() { return dm; }
}

#endif

