/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 * NOTES: time-critical routines do NO strong error control, i.e. they do
 * not raise exceptions, this may be wrong.
 * Does _NOT_ provide a reliable realtime recording mode, so recording
 * functions are provided only to be used in non time-critical tasks (
 * they may not work even in those cases ). 
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <endian.h>
#include <esd.h>
#include <unistd.h>
#include <string>
#include "esdout.h"

#include "i18n.h"

namespace Tuner {
	const int NoObjects = -2;
	const int BadFd = -1;

	const char *Hostname = 0;
	const char *Name = "GuiTuner";
}

int Tuner::Esd::fd = NoObjects;

Tuner::Esd::Esd() throw(Error::AudioError)
{
	if(fd == NoObjects) {
		fd = BadFd;
		info.loadDefaults();
	} else {
		throw Error::AudioError(std::string("Tuner::Esd::Esd"),
			Error::BAD_REQUEST,
			std::string(_("more than one Esd driver created")));
	}
}

Tuner::Esd::~Esd()
{
	if(fd != BadFd)
		::close(fd);
	fd = NoObjects;
}

bool
Tuner::Esd::openDevice(AudioInfo *ai, bool, bool) throw(Error::AudioError)
{
	esd_format_t flags = readFormat(ai);
	switch(ai->mode) {
		case PLAYBACK:
			flags |= ESD_PLAY;
			fd = esd_play_stream_fallback(flags, ai->rate,
				Hostname, Name);
			break;
		case RECORD:
			flags |= ESD_RECORD;
			fd = esd_record_stream_fallback(flags, ai->rate,
				Hostname, Name);
			break;
		case DUPLEX:
			throw Error::AudioError(
				std::string("Tuner::Esd::openDevice"),
				Error::BAD_REQUEST,
				std::string(
				_("This esd driver doesn' t support duplex mode"))
				);
	}
	ai->fragNumber = 1;
	ai->fragSize = ESD_BUF_SIZE;

	if(fd == -1)
		return false;

	info = *ai;

	return true;
}

bool
Tuner::Esd::closeDevice() throw()
{
	::close(fd), fd = BadFd;

	return true;
}

bool
Tuner::Esd::play(unsigned char *buf, int len) throw(Error::AudioError)
{
	return ::write(fd, buf, len) != -1;
}

bool
Tuner::Esd::record(unsigned char *buf, int len) throw(Error::AudioError)
{
	int recorded = 0, last, step = 0;
	while(recorded < len - ESD_BUF_SIZE) {
		if((last = ::read(fd, buf + recorded, ESD_BUF_SIZE)) == -1)
			return false;
		recorded += last;
		++step;
	}
	::read(fd, buf + recorded, len - recorded);
	
	return true;
}

bool
Tuner::Esd::avail(int *frags, int *bytes) throw(Error::AudioError)
{
	*frags = 1;
	*bytes = ESD_BUF_SIZE;

	return true;
}

bool
Tuner::Esd::free(int *frags, int *bytes) throw(Error::AudioError)
{
	*frags = 1;
	*bytes = ESD_BUF_SIZE;

	return true;
}

bool
Tuner::Esd::sync() throw(Error::AudioError)
{
	return false;
}

bool
Tuner::Esd::stop() throw(Error::AudioError)
{
	return false;
}


esd_format_t
Tuner::Esd::readFormat(AudioInfo *ai)
{
	esd_format_t r = ESD_STREAM;
	switch(ai->fmt) {
		case U8:
			r |= ESD_BITS8;
			break;
		case S16BE:
		case S16LE:
			r |= ESD_BITS16;
#if __BYTE_ORDER == __BIG_ENDIAN
			ai->fmt = S16BE;
#endif
#if __BYTE_ORDER == __LITTLE_ENDIAN
			ai->fmt = S16LE;
#endif
			break;
	}
	switch(ai->chn) {
		case MONO:
			r |= ESD_MONO;
			break;
		case STEREO:
			r |= ESD_STEREO;
			break;
	}
	return r;
}

