/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 * SEE EXCEPTIONS & SPEC
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <cstring>
#include <fcntl.h>
#include <unistd.h>
#include "genconf.h"
#include "i18n.h"

namespace Config {
	const char *EnvConfFileEntry = "GUITUNERRC";
	const char *DefaultRCName = "/.guitunerrc";
	const char *DefaultGlobalRC[2] = { "/usr/local/etc/guitunerrc",
		"/etc/guitunerrc" };

	const char *argList = "c:d:m:p:r:";
}

Config::GenConf::GenConf(int &argc, char **&argv) throw(Error::ConfError):
	Tuner::Conf(Tuner::OSS_DRIVER, Tuner::FFTCALC),
	thPtr(0)
{
	ConfFile cf;
	try {
		parseEnv();
		cf.load(inConf.c_str());
		load(cf);
	} catch(...) {
		loadDefaults();
	}
	parseCmdLine(argc, argv);
}

/*
 * Always Save ?
 */
Config::GenConf::~GenConf()
{
	save();
	if(thPtr)
		delete thPtr;
}

Thread::MainThread *
Config::GenConf::createMainThread(Interface::Gui *gui) throw(std::bad_alloc)
{
	if(thPtr)
		delete thPtr;

	sync();

	Tuner::Audio *dev = createAudio();

	if(!dev->openDevice(&getAudioInfo(), false, true))
		gui->errorMsg(_("Cannot open audio device, retry later"));

	return thPtr = new Thread::MainThread(gui, dev,
		createCalc(), createBuf(), getBufSize(), 250);
}

void
Config::GenConf::save() throw(Error::ConfError)
{
	ConfFile cf;
	try {
		Conf::save(cf);
		cf.save(outConf.c_str());
	} catch(...) {
	}
}

void
Config::GenConf::parseCmdLine(int &argc, char **&argv) throw(Error::ConfError)
{
	char ch;
	opterr = 0;

	while((ch = getopt(argc, argv, argList)) != -1) {
		switch(ch) {
			case 'c':
				if(std::strcasecmp(optarg, "en") ||
					std::strcasecmp(optarg, "english"))
					note.conv = Tuner::ENGLISH;
				if(std::strcasecmp(optarg, "it") ||
					std::strcasecmp(optarg, "italian"))
					note.conv = Tuner::ITALIAN;
				break;
			case 'd':
				if(std::strcasecmp(optarg, "oss"))
					audr = Tuner::OSS_DRIVER;
				if(std::strcasecmp(optarg, "esd"))
					audr = Tuner::ESD_DRIVER;
				break;
			case 'm':
				if(std::strcasecmp(optarg, "fft"))
					dm = Tuner::FFTCALC;
				break;
			case 'p':
				if(int p = std::atoi(optarg) > 0)
					fft.points = p;
				break;
			case 'r':
				if(int p = std::atoi(optarg) > 0)
					audio.rate = p;
				break;

			default:;
		}
	}

	update();
}

void
Config::GenConf::parseEnv() throw(Error::ConfError)
{
	char *ptr = getenv(EnvConfFileEntry);
	if(ptr == 0) {
		ptr = getenv("HOME");
		inConf = outConf = std::string(ptr) + DefaultRCName;
	} else {
		inConf = outConf = ptr;
	}

	int fd = open(inConf.c_str(), O_RDONLY);
	if(fd == -1) {
		for(int i = 0; i < 2 && fd == -1; ++i)
			fd = open((inConf = DefaultGlobalRC[i]).c_str(), O_RDONLY);
	}

	if(fd == -1)
		throw Error::ConfError("Config::GenConf::parseEnv",
			_("Cannot find configuration file"));

	close(fd);
}

