/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "error.h"
#include <cstring>
#include "mainthread.h"

Thread::MainThread::MainThread(Interface::Gui *g, Tuner::Audio *a,
	Tuner::Calc *c, unsigned char *b, int s, int refresh) throw(std::bad_alloc):
	gui(g), audioDev(a), calc(c), buffer(b), bufSize(s), paused(false)
{
	if(audioDev) {
		const Tuner::AudioInfo *ai = audioDev->getInfo();
		if(ai->fragNumber != -1)
			auxBuf = new unsigned char[auxBufSize = ai->fragNumber * ai->fragSize];
		else
			auxBufSize = 0;
		refreshBytes = static_cast<int>(refresh * ai->sampleSize() * ai->rate / 1000.0);
	} else {
		auxBuf = 0;
		auxBufSize = 0;
		refreshBytes = 0;
	}
}

Thread::MainThread::~MainThread()
{
	if(auxBuf)
		delete []auxBuf;
}

void
Thread::MainThread::reconfigure(Interface::Gui *g, Tuner::Audio *a, Tuner::Calc *c,
	unsigned char *b, int s, int refresh) throw(std::bad_alloc)
{
	if(!isRunning() || paused) {
		if(g != 0)
			gui = g;
		if(a != 0) {
			audioDev = a;
			const Tuner::AudioInfo *ai = audioDev->getInfo();
			if(auxBuf)
				delete []auxBuf;
			auxBuf = new unsigned char[auxBufSize = ai->fragNumber * ai->fragSize];
			refreshBytes = static_cast<int>(refresh * ai->sampleSize() * ai->rate /
				1000.0);
		}
		if(c != 0)
			calc = c;
		if(b != 0)
			buffer = b, bufSize = s;
	}
}

void
Thread::MainThread::adjustBuffers(int av) throw()
{
	if(av < bufSize) {
		std::memmove(buffer, buffer + av, bufSize - av);
		std::memcpy(buffer + bufSize - av, auxBuf, av);
	} else {
		std::memcpy(buffer, auxBuf + av - bufSize, bufSize); 
	}
}

int
Thread::MainThread::main() throw()
{
	using namespace Tuner;

	const AudioInfo *ai = audioDev->getInfo();
	int errorCount = 0;
	bool first = true;

	while(errorCount < 2) {
		try {
			if(pending(FROM_PARENT)) {
				Command c;
				getMsg(&c, sizeof(Command), FROM_PARENT);
				switch(c) {
					case STOP:
						gui->display(Tuner::Note(0));
						return errorCount;
					case PAUSE:
						gui->display(Tuner::Note(0));
						paused = first = true;
						break;
					case CONTINUE:
						paused = false;
				}
			}
			if(!paused) {
				if(!first) {
					int fn, b, av;

					if(!audioDev->avail(&fn, &b))
						av = refreshBytes;
					else
						av = fn * ai->fragSize;
					if(av < refreshBytes) {
						gui->delay(static_cast<double>(refreshBytes - av) *
							static_cast<double>(1000000) /
							static_cast<double>(ai->sampleSize()) /
							static_cast<double>(ai->rate));
						continue;
					}
					if(!audioDev->record(auxBuf, av)) {
						paused = true;
						continue;
					}
					adjustBuffers(av);
				} else {
					if(!audioDev->record(buffer, bufSize)) {
						paused = true;
						continue;
					}
					first = false;
				}
				gui->display(Tuner::Note((*calc)(buffer)));
			} else
				gui->delay(10000);
		} catch(Error::Error &e) {
			gui->errorMsg(e.getFullMsg());
			++errorCount;
		} catch(std::exception &e) {
			gui->errorMsg(e.what());
			++errorCount;
		}
	}

	return errorCount;
}

