/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <cstring>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include "posixthread.h"
#include "i18n.h"

const Thread::PosixThread::Mutex Thread::PosixThread::MutexInitializer = PTHREAD_MUTEX_INITIALIZER;
const Thread::PosixThread::Condition Thread::PosixThread::ConditionInitializer = PTHREAD_COND_INITIALIZER;


void *
Thread::PosixThread::mainCallback(void *ptr)
{
	void *val = reinterpret_cast<void *>(static_cast<PosixThread *>(ptr)->main());
	static_cast<PosixThread *>(ptr)->running = false;

	return val;
}

bool
Thread::PosixThread::pending(Direction from) throw()
{
	lock(bufLock[from]);
	bool rval = !buffer[from].empty();
	unlock(bufLock[from]);

	return rval;
}

bool
Thread::PosixThread::sendMsg(void *buf, int len, Direction to) throw(Error::ThreadError)
{
	if(!running)
		throw Error::ThreadError(std::string("Thread::PosixThread::sendMsg"),
			std::string(_("Communication with ghost thread")));

	char *ptr = static_cast<char *>(buf);
	int i, index;

	switch(to) {
		case TO_CHILD:
			index = P2C;
			break;
		case TO_PARENT:
			index = C2P;
			break;
		default:
			return false;
	}

	lock(bufLock[index]);

	for(i = 0; i < len; ++i)
		buffer[index].push(ptr[i]);

	signal(bufWrite[index]);
	unlock(bufLock[index]);

	return true;
}

bool
Thread::PosixThread::getMsg(void *buf, int len, Direction from) throw(Error::ThreadError)
{
	if(!running)
		throw Error::ThreadError(std::string("Thread::PosixThread::getMsg"),
			std::string(_("Communication with ghost thread")));

	char *ptr = static_cast<char *>(buf);
	int i, index;

	switch(from) {
		case FROM_CHILD:
			index = C2P;
			break;
		case FROM_PARENT:
			index = P2C;
			break;
		default:
			return false;
	}

	lock(bufLock[index]);

	while(buffer[index].size() < len)
		wait(bufLock[index], bufWrite[index]);

	for(i = 0; i < len; ++i) {
		ptr[i] = buffer[index].front();
		buffer[index].pop();
	}

	unlock(bufLock[index]);

	return true;
}

Thread::PosixThread::PosixThread() throw(Error::ThreadError): running(false)
{
	for(int i = 0; i < BUF_NUM; ++i) {
		bufLock[i] = MutexInitializer;
		bufWrite[i] = ConditionInitializer;
	}
}

Thread::PosixThread::~PosixThread()
{
	if(running) {
		pthread_cancel(threadInfo);
		pthread_join(threadInfo, 0);
	}
}

bool
Thread::PosixThread::run() throw(Error::ThreadError)
{
	if(pthread_create(&threadInfo, 0, PosixThread::mainCallback, this))
		throw Error::ThreadError(std::string("Thread::PosixThread::run"),
			std::string(_("Cannot start thread")));
	running = true;

	return true;
}

bool
Thread::PosixThread::reset() throw(Error::ThreadError)
{
	if(!running) {
		for(int i = 0; i < BUF_NUM; ++i)
			while(!buffer[i].empty())
				buffer[i].pop();
		return true;
	}

	return false;
}

bool
Thread::PosixThread::kill() throw(Error::ThreadError)
{
	pthread_cancel(threadInfo);
	pthread_join(threadInfo, 0);
	running = false;

	return true;
}

bool
Thread::PosixThread::wait() throw(Error::ThreadError)
{
	pthread_join(threadInfo, 0);
	running = false;

	return true;
}

