/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 * Call wait() for EVERY thread, even if it's over
 */

#ifndef _POSIXTHREAD_H_
#define _POSIXTHREAD_H_

#include <pthread.h>
#include <queue>
#include <unistd.h>
#include "thread.h"

namespace Thread {
	enum Direction {
		TO_CHILD,
		TO_PARENT,
		FROM_CHILD = TO_PARENT,
		FROM_PARENT = TO_CHILD
	};

	enum {
		P2C,
		C2P,
		BUF_NUM
	};

	class PosixThread: public BasicThread {
		public:
		typedef pthread_mutex_t Mutex;
		typedef pthread_cond_t Condition;

		static const Mutex MutexInitializer;
		static const Condition ConditionInitializer;

			PosixThread() throw(Error::ThreadError);
		    ~PosixThread();

		int	main() throw() = 0;
		bool	run() throw(Error::ThreadError);

		bool	pending() throw();
		bool	sendMsg(void *, int) throw(Error::ThreadError);
		bool	getMsg(void *, int) throw(Error::ThreadError);
		bool	reset() throw(Error::ThreadError);
		bool	kill() throw(Error::ThreadError);
		bool	wait() throw(Error::ThreadError);

		bool	lock(Mutex &) throw();
		bool	unlock(Mutex &) throw();

		bool	wait(Mutex &, Condition &) throw();
		bool	signal(Condition &) throw();

		bool	isRunning() throw();

		protected:
		bool	pending(Direction) throw();
		bool	sendMsg(void *, int, Direction) throw(Error::ThreadError);
		bool	getMsg(void *, int, Direction) throw(Error::ThreadError);

		private:
		static	void	*mainCallback(void *);

		std::queue<char> buffer[BUF_NUM];
		Mutex bufLock[BUF_NUM];
		Condition bufWrite[BUF_NUM];
		pthread_t threadInfo;
		bool running;
	};

	inline bool
	PosixThread::pending() throw()
	{ return pending(FROM_CHILD); }

	inline bool
	PosixThread::sendMsg(void *ptr, int len) throw(Error::ThreadError)
	{ return sendMsg(ptr, len, TO_CHILD); }

	inline bool
	PosixThread::getMsg(void *ptr, int len) throw(Error::ThreadError)
	{ return getMsg(ptr, len, FROM_CHILD); }

	inline bool
	PosixThread::lock(Mutex &m) throw() { return pthread_mutex_lock(&m), true; }

	inline bool
	PosixThread::unlock(Mutex &m) throw() { return pthread_mutex_unlock(&m), true; }

	inline bool
	PosixThread::wait(Mutex &m, Condition &c) throw() { return pthread_cond_wait(&c, &m), true; }

	inline bool
	PosixThread::signal(Condition &c) throw() { return pthread_cond_signal(&c), true; }

	inline bool
	PosixThread::isRunning() throw() { return running; }

	class Resource {
		public:
			Resource() throw();

		private:
		friend class Lock;
		int val;
	};

	class Lock {
		public:
			Lock(Resource &) throw();
			~Lock();

		void unlock() throw();

		private:
		Resource &res;
		bool locked;
	};

	inline Resource::Resource() throw() { val = -1; }

	inline Lock::Lock(Resource &r) throw(): res(r), locked(true)
	{
		if(++(res.val)) {
			while(static_cast<volatile int>(res.val))
				usleep(50);
		}
	}

	inline void Lock::unlock() throw() { --(res.val), locked = false; }

	inline Lock::~Lock() { if(locked) unlock(); }
}

#endif

